"{ Package: 'jv:calipel/s' }"

Object subclass:#BenchmarkRunner
	instanceVariableNames:'suite result'
	classVariableNames:''
	poolDictionaries:''
	category:'CalipeL-S-Core-Runner'
!

!BenchmarkRunner class methodsFor:'documentation'!

documentation
"
    BechmarkRunner is the command line client for CalipeL.
    It parses the command line arguments, sets up the
    environment and run benchmarks. 
"
! !

!BenchmarkRunner class methodsFor:'instance creation'!

new
    "return an initialized instance"

    ^ self basicNew initialize.
! !

!BenchmarkRunner methodsFor:'accessing'!

result
    ^ result
!

suite
    ^ suite
! !

!BenchmarkRunner methodsFor:'main'!

main:argv0
    | i report machineid file params classes runs argv |

    params := Dictionary new.
    classes := OrderedCollection new.
    report := BenchmarkReport text.
    runs := 5.
    argv := argv0 asOrderedCollection.

    i := 1.
    [ i <= argv size ] whileTrue:[
        | arg |

        arg := argv at: i.
        i := i + 1.
        arg first == $- ifTrue:[
            arg = '-n' ifTrue:[
                i > argv size ifTrue:[
                    self error:'-n requires an integer parameter'
                ].
                runs := Integer readFrom: (argv at: i) onError:[
                    self error: '-n requires an integer parameter'.
                ].
                i := i + 1.
            ].
            arg = '-o' ifTrue:[
                i > argv size ifTrue:[
                    self error:'-o requires a filename parameter'
                ].
                file := argv at: i.
                i := i + 1.
            ].
            arg = '--arguments' ifTrue:[
                | arguments argumentsF |

                i > argv size ifTrue:[
                    self error:'--arguments requires a file parameter.'
                ].                
                argumentsF := (arguments := (argv at: i)) asFilename.
                argumentsF exists ifFalse:[
                    self error:'no such file: ', arguments.
                ].
                argumentsF readingFileDo:[:s|
                    [ s atEnd ] whileFalse:[
                        argv add: s nextLine.
                    ]
                ].
                i := i + 1.
            ].  
                         

            arg = '-r' ifTrue:[
                | reportNm |

                i > argv size ifTrue:[
                    self error:'-r requires a valid class name parameter'
                ].
                reportNm := (argv at: i).
                i := i + 1.
                report := Smalltalk at: reportNm asSymbol.
                report isNil ifTrue:[
                    self error: 'No report class named ''', reportNm, ''''.
                ].
            ].
            arg = '--text' ifTrue:[
                report := BenchmarkReport text.
            ].

            arg = '--json' ifTrue:[
                report := BenchmarkReport json.
            ].

            arg = '--machineid' ifTrue:[
                i > argv size ifTrue:[
                    self error:'--machineid requires a string parameter.'
                ].                
                machineid := argv at: i.
                i := i + 1.
            ].

            arg second = $D ifTrue:[
                | eqIdx |

                ((arg size > 2) and:[(eqIdx := arg indexOf: $= startingAt: 3) ~~ 0]) ifTrue:[
                    params at: (arg copyFrom: 3 to: eqIdx - 1) put: (arg copyFrom: eqIdx + 1)
                ] ifFalse:[
                    self error: 'No parameter value'
                ]
            ]
        ] ifFalse:[
            classes add: arg.
        ]
    ].


    classes isEmpty ifTrue:[
        self error:'No suite or benchmark specified.'
    ].

    "Build suite"
    suite := BenchmarkSuite new.
    classes do:[:each|
        | classNm class selector |    
        i := each indexOf: $#.
        i ~~ 0 ifTrue:[
            classNm := each copyTo: i - 1.
            selector := (each copyFrom: i + 1) asSymbol.
        ] ifFalse:[
            classNm := each.
        ].
        class := Smalltalk at: classNm asSymbol.
        class isNil ifTrue:[
            self error: 'Class ', classNm , ' does not exists'.
        ].
        Smalltalk isSmalltalkX ifTrue:[
            class isLoaded ifFalse:[class autoload].
        ].
        selector isNil ifTrue:[
            suite addBenchmark: (BenchmarkSuite class: class)
        ] ifFalse:[
            suite addBenchmark: (BenchmarkSuite class:class selector:selector)
        ]
    ].

    "Run suite"
    result := BenchmarkResult new.
    result runs: runs.
    suite run: result with: params executor: BenchmarkRunnerExecutor new.

    "Write report"
    file notNil ifTrue:[
        file asFilename writingFileDo:[:s|
            report
                machineId: machineid;
                write: result on: s 
        ]
    ] ifFalse:[
        report
            machineId: machineid;
            write: result on: BenchmarkPlatform current stdout
    ].

    "Modified: / 01-08-2013 / 18:50:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
!

options
    "Prints common command line options to stderr"

    BenchmarkPlatform current stderr nextPutAll:
'
Common options:
  -o FILE ................ write output to FILE instead of to standard output (default: stdout)
  -n RUNS ................ how many times to run each bechmark (default: 5)
  -r REPORTCLASS ......... user REPORTCLASS to generate report (default: BenchmarkReportText)
  --arguments FILE ....... read additional arguments from FOLE, one argument per line.
  --text ................. generate text report (equivalent to -r BenchmarkReportText)
  --json ................. generate JSON report (equivalent to -r BenchmarkReportJSON)
  --machineid ID ......... set the machine idetification string for JSON report
  -DNAME=VALUE ........... defines a parameter NAME with value of VALUE

Suitespec format:
  <suitespec> is CLASSNAME[#SELECTOR]

More documetation:
  see https://bitbucket.org/janvrany/jv-calipel/wiki/Home

'

    "Created: / 06-06-2013 / 11:01:42 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 31-07-2013 / 00:46:15 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!BenchmarkRunner class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

