class FileImportQuery < ImportQuery

  attr_accessor :file

  validates_presence_of :file 

  def data_from_json
    @parsed_json ||= JSON.parse(file.read)
  end

  def store!
    super
    store_to_filesystem!
  end

  private

  def store_to_filesystem!
    pretty_printed_json = JSON.pretty_generate(data_from_json)
    digest = Digest::MD5.hexdigest(pretty_printed_json)

    File.open(nice_path(digest), "w") do |f|
      f.write(pretty_printed_json)
    end
  end

  def nice_path(digest)
    pathname = Pathname.new(file.original_filename)
    basename = pathname.basename(pathname.extname)

    Rails.root.
      join("public").
      join("uploads").
      join("#{Time.now.to_i}_#{basename}_#{digest}.json")
  end


end
